@@ -29,7 +29,11 @@ from .patterns import \
     InstanceOf, \
     IsNumber, \
     IsString, \
+    IsTruish, \
     Length, \
+    Maybe, \
+    NoneOf, \
+    Object, \
     Regex, \
     Returns, \
     Transformed, \
@@ -71,7 +75,11 @@ __all__ = [
     'InstanceOf',
     'IsNumber',
     'IsString',
+    'IsTruish',
     'Length',
+    'Maybe',
+    'NoneOf',
+    'Object',
     'Regex',
     'Returns',
     'Transformed',
