@@ -1,12 +1,26 @@
 from __future__ import annotations
 
+from collections import Mapping  # pylint: disable=no-name-in-module
 from itertools import chain
-from typing import Optional
+from typing import Optional, List, Dict
+
+
+class WildcardMatch:
+    def __init__(self, index):
+        self.index = index
+        self.value = None
+
+    def set(self, value):
+        self.value = value
+
+    def get(self):
+        return self.value
 
 
 class MatchContext:
     def __init__(self, *, multimatch: bool, strict: bool):
         self.groups = {}
+        self.wildcards: Dict[int, WildcardMatch] = {}
         self.multimatch = multimatch
         self.strict = strict
 
@@ -61,11 +75,21 @@ class MatchContext:
     def match_if(self, condition: bool) -> MatchResult:
         return MatchResult(matches=bool(condition), context=self)
 
+    def record(self, for_pattern, value):
+        id_ = id(for_pattern)
+        if id_ not in self.wildcards:
+            self.wildcards[id_] = WildcardMatch(len(self.wildcards))
+        self.wildcards[id_].set(value)
 
-class MatchResult:
+
+class MatchResult(Mapping):
     def __init__(self, *, matches: bool, context: MatchContext):
         self._matches: bool = matches
         self._context: MatchContext = context
+        self._wildcard_matches: List = [None] * len(context.wildcards)
+
+        for wildcard_match in context.wildcards.values():
+            self._wildcard_matches[wildcard_match.index] = wildcard_match.value
 
     def __bool__(self):
         return self._matches
@@ -82,9 +106,18 @@ class MatchResult:
     def __iter__(self):
         return iter(self._context.groups)
 
+    def __len__(self):
+        return len(self._context.groups)
+
     def items(self):
         return self._context.groups.items()
 
+    def groups(self) -> Dict:
+        return self._context.groups
+
+    def wildcard_matches(self) -> List:
+        return self._wildcard_matches
+
 
 class Capturable:
     def __rshift__(self, other):
@@ -221,6 +254,7 @@ class Value(Pattern):
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         if self._value is Ellipsis:
+            ctx.record(self, value)
             return ctx.matches()
         if strict:
             if type(self._value) != type(value):
