@@ -1,3 +1,6 @@
+from itertools import chain
+from typing import Union, Any
+
 from .core import MatchResult, MatchContext
 from .try_match import TryMatch
 
@@ -20,13 +23,24 @@ class NoValue:
     pass
 
 
-def match(value, pattern=NoValue, *, multimatch=True, strict=False, argresult=False) -> MatchResult:
+def match(value, pattern=NoValue, *extra, multimatch=True, strict=False, argresult=False) -> Union[MatchResult, Any]:
     ctx = MatchContext(
         multimatch=multimatch,
         strict=strict,
     )
     if pattern is NoValue:
         raise TryMatch(value, ctx=ctx)
+    elif extra:
+        acc = []
+        for p in chain((pattern,), extra):
+            acc.append(p)
+            if len(acc) == 2:
+                condition, action = acc
+                if result := match(value, condition):
+                    if callable(action):
+                        return action(*result.wildcard_matches())
+                    return action
+                acc = []
     result = ctx.match(value, pattern, strict=strict)
     if argresult:
         result = AttributesAdapter(result)
