@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import unittest
+from datetime import datetime
 
 from apm import *
 
@@ -43,6 +44,10 @@ class BasicUseCases(unittest.TestCase):
         self.assertTrue(match([], Length(0)))
         self.assertTrue(match(('abc',), Length(1)))
         self.assertTrue(match('abc', Length(3)))
+        self.assertTrue(match('abc', Length(at_least=1, at_most=4)))
+        self.assertFalse(match('abc', Length(at_least=4, at_most=10)))
+        self.assertTrue(match('abcde', Length(at_least=4, at_most=5)))
+        self.assertFalse(match('abcdef', Length(at_least=4, at_most=5)))
 
     def test_contains(self):
         self.assertTrue(match([1, 2, 3], Contains(2) & Contains(3)))
@@ -199,6 +204,15 @@ class BasicUseCases(unittest.TestCase):
         self.assertFalse(match(g, Arguments(int)))
         self.assertTrue(match(h, Arguments(None, int, Remaining(str))))
 
+    def test_keyword_arguments(self):
+        # noinspection PyUnusedLocal
+        def f(x: int, y: bool):
+            pass
+
+        self.assertTrue(match(f, Arguments(x=int)))
+        self.assertFalse(match(f, Strict(Arguments(x=int))))
+        self.assertTrue(match(f, Strict(Arguments(x=int, y=bool))))
+
     def test_returns(self):
         def f() -> int:
             pass
@@ -270,3 +284,62 @@ class BasicUseCases(unittest.TestCase):
         self.assertTrue(match(1, IsNumber))
         self.assertTrue(match(1.0, IsNumber))
         self.assertFalse(match(True, IsNumber))
+
+    def test_object(self):
+        request = {
+            "api_version": "v1",
+            "job": {
+                "run_at": "2020-08-27 14:09:30",
+                "command": "echo 'booya'",
+            }
+        }
+
+        # noinspection PyUnresolvedReferences
+        self.assertTrue(result := match(request, Object(
+            api_version="v1",
+            job=Object(
+                run_at=Transformed(datetime.fromisoformat, 'time' @ _),
+            ) & OneOf(
+                Object(command='command' @ InstanceOf(str)),
+                Object(spawn='container' @ InstanceOf(str)),
+            )
+        )))
+        self.assertFalse('container' in result)
+        self.assertEqual(result['time'], datetime(2020, 8, 27, 14, 9, 30))
+        self.assertEqual(result['command'], "echo 'booya'")
+
+    def test_transformed_exception(self):
+        request = {
+            "api_version": "v1",
+            "job": {
+                "run_at": "junk",
+                "command": "echo 'booya'",
+            }
+        }
+
+        # noinspection PyUnresolvedReferences
+        self.assertFalse(match(request, Object(
+            job=Object(
+                run_at=Transformed(datetime.fromisoformat, 'time' @ _),
+            )
+        )))
+
+    def test_splat_result(self):
+        def f(**kwargs):
+            dct = {}
+            for k, v in kwargs.items():
+                dct[k] = v
+            return dct
+
+        self.assertTrue(result := match({'a': 1, 'b': 2, 'c': 3}, Object(a=_ >> 'x', b=_ >> 'y', c=_ >> 'z')))
+        r = f(**result)
+        self.assertEqual({'x': 1, 'y': 2, 'z': 3}, r)
+
+    def test_maybe(self):
+        pattern = Object(a=...) & Maybe(Object(b=_ >> 'capture'))
+
+        self.assertTrue(result := match({'a': 3}, pattern))
+        self.assertFalse('capture' in result)
+        self.assertTrue(result := match({'a': 3, 'b': 5}, pattern))
+        self.assertTrue('capture' in result)
+        self.assertEqual(5, result['capture'])
